@@ -13,6 +13,8 @@ from spacepackets.ccsds.spacepacket import (
     SPACE_PACKET_HEADER_SIZE,
     SpacePacket,
     PacketId,
+    PacketSeqCtrl,
+    SequenceFlags,
 )
 from spacepackets.util import get_printable_data_string, PrintFormats
 from spacepackets.ecss.conf import (
@@ -24,7 +26,7 @@ from spacepackets.ecss.conf import (
 
 
 try:
-    from crcmod.predefined import mkPredefinedCrcFun
+    from crcmod.predefined import mkPredefinedCrcFun, PredefinedCrc
 except ImportError:
     print("crcmod package not installed!")
     sys.exit(1)
@@ -94,6 +96,9 @@ class PusTcDataFieldHeader:
             f" ack_flags={self.ack_flags!r} "
         )
 
+    def __eq__(self, other: PusTcDataFieldHeader):
+        return self.pack() == other.pack()
+
     @classmethod
     def get_header_size(cls):
         return cls.PUS_C_SEC_HEADER_LEN
@@ -110,7 +115,8 @@ class PusTelecommand:
         service: int,
         subservice: int,
         app_data: bytes = bytes([]),
-        ssc: int = 0,
+        seq_count: int = 0,
+        seq_flags: SequenceFlags = SequenceFlags.UNSEGMENTED,
         source_id: int = 0,
         ack_flags: int = 0b1111,
         apid: int = FETCH_GLOBAL_APID,
@@ -121,7 +127,7 @@ class PusTelecommand:
         :param service: PUS service number
         :param subservice: PUS subservice number
         :param apid: Application Process ID as specified by CCSDS
-        :param ssc: Source Sequence Count. Application should take care of incrementing this.
+        :param seq_count: Source Sequence Count. Application should take care of incrementing this.
             Limited to 2 to the power of 14 by the number of bits in the header
         :param app_data: Application data in the Packet Data Field
         :param source_id: Source ID will be supplied as well. Can be used to distinguish
@@ -150,26 +156,52 @@ class PusTelecommand:
             apid=apid,
             sec_header_flag=bool(secondary_header_flag),
             packet_type=PacketTypes.TC,
+            seq_flags=seq_flags,
             data_len=data_length,
-            seq_count=ssc,
+            seq_count=seq_count,
         )
         self._app_data = app_data
         self._valid = True
-        self._crc = 0
+        self._crc16 = 0
+
+    @classmethod
+    def from_sp_header(
+        cls,
+        sp_header: SpacePacketHeader,
+        service: int,
+        subservice: int,
+        app_data: bytes = bytes([]),
+        source_id: int = 0,
+        ack_flags: int = 0b1111,
+    ):
+        pus_tc = cls.__empty()
+        if sp_header.packet_type == PacketTypes.TM:
+            raise ValueError(
+                f"Invalid Packet Type {sp_header.packet_type} in CCSDS primary header"
+            )
+        pus_tc.sp_header = sp_header
+        pus_tc.pus_tc_sec_header = PusTcDataFieldHeader(
+            service=service,
+            subservice=subservice,
+            source_id=source_id,
+            ack_flags=ack_flags,
+        )
+        pus_tc._app_data = app_data
+        return pus_tc
 
     @classmethod
     def from_composite_fields(
         cls,
-        sph: SpacePacketHeader,
+        sp_header: SpacePacketHeader,
         sec_header: PusTcDataFieldHeader,
         app_data: bytes = bytes([]),
     ) -> PusTelecommand:
         pus_tc = cls.__empty()
-        if sph.packet_type == PacketTypes.TM:
+        if sp_header.packet_type == PacketTypes.TM:
             raise ValueError(
-                f"Invalid Packet Type {sph.packet_type} in CCSDS primary header"
+                f"Invalid Packet Type {sp_header.packet_type} in CCSDS primary header"
             )
-        pus_tc.sp_header = sph
+        pus_tc.sp_header = sp_header
         pus_tc.pus_tc_sec_header = sec_header
         pus_tc._app_data = app_data
         return pus_tc
@@ -181,7 +213,7 @@ class PusTelecommand:
     def __repr__(self):
         """Returns the representation of a class instance."""
         return (
-            f"{self.__class__.__name__}.from_composite_fields(sph={self.sp_header!r}, "
+            f"{self.__class__.__name__}.from_composite_fields(sp_header={self.sp_header!r}, "
             f"sec_header={self.pus_tc_sec_header!r}, app_data={self.app_data!r})"
         )
 
@@ -189,30 +221,50 @@ class PusTelecommand:
         """Returns string representation of a class instance."""
         return (
             f"PUS TC[{self.pus_tc_sec_header.service}, "
-            f"{self.pus_tc_sec_header.service}], APID {self.apid:#05x}, "
+            f"{self.pus_tc_sec_header.subservice}], APID {self.apid:#05x}, "
             f"SSC {self.sp_header.seq_count}, Size {self.packet_len}"
         )
 
-    def to_space_packet(self):
-        """Retrieve generic CCSDS space packet"""
+    def __eq__(self, other: PusTelecommand):
+        return (
+            self.sp_header == other.sp_header
+            and self.pus_tc_sec_header == other.pus_tc_sec_header
+            and self._app_data == other._app_data
+        )
+
+    def to_space_packet(self) -> SpacePacket:
+        """Retrieve the generic CCSDS space packet representation. This also calculates the CRC16
+        before converting the PUS TC to a generic Space Packet"""
+        self.calc_crc()
         user_data = bytearray(self._app_data)
-        user_data.extend(struct.pack("!H", self._crc))
+        user_data.extend(struct.pack("!H", self._crc16))
         return SpacePacket(self.sp_header, self.pus_tc_sec_header.pack(), user_data)
 
     @property
     def valid(self):
         return self._valid
 
-    def pack(self) -> bytearray:
-        """Serializes the TC data fields into a bytearray."""
+    def calc_crc(self):
+        """Can be called to calculate the CRC16"""
+        crc = PredefinedCrc(crc_name="crc-ccitt-false")
+        crc.update(self.sp_header.pack())
+        crc.update(self.pus_tc_sec_header.pack())
+        crc.update(self.app_data)
+        self._crc16 = crc.crcValue
+
+    def pack(self, calc_crc: bool = True) -> bytearray:
+        """Serializes the TC data fields into a bytearray.
+        :param calc_crc: Recalculate the CRC. Can be disabled if :py:func:`calc_crc`
+            was called before
+        """
         packed_data = bytearray()
         packed_data.extend(self.sp_header.pack())
         packed_data.extend(self.pus_tc_sec_header.pack())
         packed_data += self.app_data
-        crc_func = mkPredefinedCrcFun(crc_name="crc-ccitt-false")
-        self._crc = crc_func(packed_data)
-        self._valid = True
-        packed_data.extend(struct.pack("!H", self._crc))
+        if calc_crc:
+            crc_func = mkPredefinedCrcFun(crc_name="crc-ccitt-false")
+            self._crc16 = crc_func(packed_data)
+        packed_data.extend(struct.pack("!H", self._crc16))
         return packed_data
 
     @classmethod
@@ -234,7 +286,7 @@ class PusTelecommand:
             )
             raise ValueError
         tc_unpacked._app_data = raw_packet[header_len : expected_packet_len - 2]
-        tc_unpacked._crc = raw_packet[expected_packet_len - 2 : expected_packet_len]
+        tc_unpacked._crc16 = raw_packet[expected_packet_len - 2 : expected_packet_len]
         crc_func = mkPredefinedCrcFun(crc_name="crc-ccitt-false")
         whole_packet = raw_packet[:expected_packet_len]
         should_be_zero = crc_func(whole_packet)
@@ -297,10 +349,18 @@ class PusTelecommand:
     def packet_id(self) -> PacketId:
         return self.sp_header.packet_id
 
+    @property
+    def packet_seq_ctrl(self) -> PacketSeqCtrl:
+        return self.sp_header.psc
+
     @property
     def app_data(self) -> bytes:
         return self._app_data
 
+    @property
+    def crc16(self):
+        return self._crc16
+
     def print(self, print_format: PrintFormats = PrintFormats.HEX):
         """Print the raw command in a clean format."""
         packet = self.pack()
